;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x3deb1ce93911dea01a5d1e15e4d50d09 . "./code.cg2js/cg-acache.fasl")
(#xa9de79ec9e1d3c633088ae0598e269e9 . "./code.cg2js/cg-base.fasl")
(#x8256f4dc66f71d5e8ff55ef78826b0ef . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x4f8ff463ac68e936c2d3270160ccb143 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x21ac470deca5ac5edc43284268803dac . "./code.cg2js/cg-bitmap-stream.fasl")
(#xab4e9b8a4d2a8bd36f86016ace00488e . "./code.cg2js/cg-button.fasl")
(#x5104a9365b7d6a1c3b5443c293fa4dca . "./code.cg2js/cg-calendar.fasl")
(#xc5a13525beb546033a25a48ddc276271 . "./code.cg2js/cg-caret.fasl")
(#x8122fd846a2d37ea808e02ad8a7590b8 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x78dc118d24f87a83257fd3f3243b148b . "./code.cg2js/cg-chart-widget.fasl")
(#xe9f42560b2853d56bc5b833bc30c7ec5 . "./code.cg2js/cg-check-box.fasl")
(#xb4e94ed5ce7a72d4ef59ade3b27423ef . "./code.cg2js/cg-choice-list.fasl")
(#x711040f59408298f40e797335cc56767 . "./code.cg2js/cg-choose-printer.fasl")
(#xacb243191fed4b59b3b55be94a46b1e8 . "./code.cg2js/cg-class-grid.fasl")
(#x8ee9ac8e691e8f3ec78552f5a65a5c89 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x5c31392285f27c4b10b9ebd25725a30d . "./code.cg2js/cg-class-support.fasl")
(#xacda258f95a8f08a12f856912e46c90f . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x6a88e371cf6189fa6793276c52337dcc . "./code.cg2js/cg-clipboard-stack.fasl")
(#x8fbeef61e3dc755bbed5aca4d904fd34 . "./code.cg2js/cg-clipboard.fasl")
(#x491a8c05cd68d989ad27b75b69b66ae7 . "./code.cg2js/cg-color-dialog.fasl")
(#xd5ee629afab3a0466bd7ac1676e26d76 . "./code.cg2js/cg-combo-box.fasl")
(#xbc52670347b9f3695c9a1c742486e94e . "./code.cg2js/cg-common-control.fasl")
(#x7bf40e796277a1860708e56e2a85d4d7 . "./code.cg2js/cg-comtab.fasl")
(#x211be33a488c4ea6d08791ddde94d09c . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xcabc9cc9498e8f159c368efbe2cb4528 . "./code.cg2js/cg-curve.fasl")
(#xf4a0fda08f5938345db814098483832f . "./code.cg2js/cg-dde-utils.fasl")
(#x004ba1f01d0738ed162b51829ca6d25c . "./code.cg2js/cg-dev.fasl")
(#x0c02367f376b5dbc569a5902ecdebd4e . "./code.cg2js/cg-dialog-item.fasl")
(#x08352e9e30dd13cee2210cb0e80b2db9 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x18298ae8bd10939a29e71017eac06c82 . "./code.cg2js/cg-directory-dialog.fasl")
(#x4021b5ab9b2b92ad6e830b1c7501f956 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x042ed09c893edbae54b51e0860a50884 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x4edf2a163cc06c17a7114c91f498052d . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xd867afb0e025e3f619fcafc4d577fbde . "./code.cg2js/cg-drawable.fasl")
(#x51703f8c22386af517ebd9154fd52cfb . "./code.cg2js/cg-dropping-outline.fasl")
(#x13a5af98547923889f21b9bbcbfc633e . "./code.cg2js/cg-edit-in-place.fasl")
(#xdbee5c1b91082a8cb8ca7562d6d311a6 . "./code.cg2js/cg-editable-text.fasl")
(#x9aab0ab848d0f0dba91e9178920f3842 . "./code.cg2js/cg-file-dialog.fasl")
(#x76130fe4b923c89558a20ed9ce491298 . "./code.cg2js/cg-fill-texture.fasl")
(#x8b05ab3683c79d91e0ebd50594d78147 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xbdf780c9765710b16739794d39046a33 . "./code.cg2js/cg-font-dialog.fasl")
(#x5554dae967a69a9640117fdf4b788982 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x068ba8863822ab7fba91b8077ca4060f . "./code.cg2js/cg-get-pixmap.fasl")
(#x84ac9238fbd32724e00ebcce577dde08 . "./code.cg2js/cg-get-position.fasl")
(#x4691d7627014747b0d7b8dc4b0794ec9 . "./code.cg2js/cg-graphics-context.fasl")
(#xd60329407189d5039b288148a29da300 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x14efc3b92fca63ff0f15b1cfe0a5292a . "./code.cg2js/cg-grid-widget.fasl")
(#x985ae22dd1745f93d0b838dd120b2256 . "./code.cg2js/cg-group-box.fasl")
(#xbe1a13eda740aef8f1a2c203fa164295 . "./code.cg2js/cg-header-control.fasl")
(#x932d0232fb19f39265ad66de27ca44af . "./code.cg2js/cg-hotspot.fasl")
(#x838905d92f06ab8162149b10196a7013 . "./code.cg2js/cg-html-dialog.fasl")
(#xd9b83221a0f7c3ae120a7dbab8443639 . "./code.cg2js/cg-html-widget.fasl")
(#xe07791ce2df47cdc9a97ee6ffde61487 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x68e063f5c5a7c2a5210e7d52899fb30b . "./code.cg2js/cg-icon.fasl")
(#xb447820d269fed739b387b73d7e3ee0e . "./code.cg2js/cg-ie.fasl")
(#x0415355569d8eaecf6c70d07bb13ce1f . "./code.cg2js/cg-intersect-posbox.fasl")
(#xe055f34bad5fee36f36cf119ff6abbc6 . "./code.cg2js/cg-intersect.fasl")
(#xd23684b418375bb83a0b12e5a0b0239a . "./code.cg2js/cg-item-list.fasl")
(#xea57e78e2ea00e7e983bbc8e67782d54 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x4ce43f318a030855e9a6c483d267579c . "./code.cg2js/cg-lamp.fasl")
(#x98b5d7fab49fbf4bc7226c0be1e41d15 . "./code.cg2js/cg-layout.fasl")
(#x5c7136a0df6785c425a1c0934af1ed15 . "./code.cg2js/cg-lettered-menu.fasl")
(#x1bd120b8d9382e197b90d9d833f2eac3 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x3028d242ab7e56f8623f469389c42f9e . "./code.cg2js/cg-lisp-text.fasl")
(#x05e8b6dd5490b2f852d7f298294de60d . "./code.cg2js/cg-lisp-widget.fasl")
(#xd5bb0774dbd9e72ffdc74e38324262a0 . "./code.cg2js/cg-list-view.fasl")
(#x22e9974d9efdab9aedb32f2668e33da4 . "./code.cg2js/cg-mci.fasl")
(#x3451c8c4cadd9e914e4627c73488aeb7 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x7d06cd194b55291fb351608be798c52a . "./code.cg2js/cg-menu.fasl")
(#xf465aade2109cd1e92dd22844208f51f . "./code.cg2js/cg-message-dialog.fasl")
(#x53ef7a8c2ba723dd49f9b6f7e02f0922 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x64abcd313b9c1c9b08e29104ae150caa . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x534754202f047fbce9d69dd1fd6b241b . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x7465d31dc30485bb52f1eb86d5e2eafb . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x2aae8b79d94f9c66eb6c02b68d22530a . "./code.cg2js/cg-multi-picture-button.fasl")
(#x5ebdf7b6c14c9e0ccdfde44aea0df646 . "./code.cg2js/cg-nodes.fasl")
(#x1ce7974d45cb2a8e38511f4421d02964 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x1d6d6741cc4257fe78536c73e19eb552 . "./code.cg2js/cg-object-editor.fasl")
(#x0de64c02745b6949fc9e7dc5f40a0834 . "./code.cg2js/cg-ocx.fasl")
(#x584aa4f04655b70d1b8432762afd458b . "./code.cg2js/cg-os-widget.fasl")
(#xb98c8a0fc5ac52b472414bce114e2e1f . "./code.cg2js/cg-os-window.fasl")
(#x46383947cf8a8a2dfe0be96f2b727793 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x53b9a0ff069c0c06be33f7d4f3d1f78f . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x836f561e4c03f17ad391f98819b65c60 . "./code.cg2js/cg-outline.fasl")
(#x8420337c924e753e102ba56cc1d488fe . "./code.cg2js/cg-palette.fasl")
(#xa69acf1890d401f9e049d5c2ac31e692 . "./code.cg2js/cg-paren-matching.fasl")
(#x698a1d1456c1fb81a2c60608880e70f3 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc8529604f47131f779c389ef4232c5c5 . "./code.cg2js/cg-picture-widget.fasl")
(#xaf7150a3b37a66f45730f2c583293429 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x48fbc43531d5ae767d916a4c97abd357 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xbe3368615f34ad5ab2fc98c6ca23fcc8 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x4d69ac196fad5b7d4870ae76027a5708 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x825bc9b123a22c9504561ae94d9669ed . "./code.cg2js/cg-pixmap.fasl")
(#xcf47be955d6a6f947e55190e79f2d5cb . "./code.cg2js/cg-plot-widget.fasl")
(#xa724949917fef9c20d293b6cab976acc . "./code.cg2js/cg-printing.fasl")
(#x12e7a147aaf127b9144c5e417d85f3e7 . "./code.cg2js/cg-progress-indicator.fasl")
(#x6b320fdd29ec8462c814585f19c73def . "./code.cg2js/cg-project-window.fasl")
(#x776f6b8eadb793bbdfb393e6310b661c . "./code.cg2js/cg-property.fasl")
(#x5819e927231415ad9f63b78dfd1557bb . "./code.cg2js/cg-radio-button.fasl")
(#xefc408dd1b75680ac4c8ff39ae9ca66d . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x80ff5e40795bcd58441f5c18712ec503 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x074fb294d85971264027dd30b6356284 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xf8284e575e3ff697c1bf6438c7212d00 . "./code.cg2js/cg-rich-edit.fasl")
(#x4a081b24ed6aa4c0cf261b293a6f56e4 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x7cbb3ffbc269bb4cd61e36e1e7bca5ce . "./code.cg2js/cg-scaling-stream.fasl")
(#xcd515a3bcf7d7e58667a299ec6894e6f . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xb383990cd013555ffa6dba23c3906298 . "./code.cg2js/cg-scroll-bar.fasl")
(#xff0384b5ad3fe5549568b5a812cb01e6 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xe32a5dbacb00beb05ceee53935ab2aab . "./code.cg2js/cg-selected-object.fasl")
(#xce6f7f9a86e0f18e8cc29ec59dcbcc9b . "./code.cg2js/cg-shortcut-menu.fasl")
(#x198dddf53d60ead553b522f3ba657f0f . "./code.cg2js/cg-split-bar.fasl")
(#x86e66f3113ee8630d9191025b4d78ade . "./code.cg2js/cg-static-text.fasl")
(#x645f841175ec53218a82c460ae60ed2c . "./code.cg2js/cg-status-bar.fasl")
(#x15381d408cc27eb138d93ad3caf32f28 . "./code.cg2js/cg-string-dialog.fasl")
(#x61636824995ec1ee47d61a8efbca7566 . "./code.cg2js/cg-tab-control.fasl")
(#x4c92bdee42173d855259d4f876119a4a . "./code.cg2js/cg-template-string.fasl")
(#xc36fd9d04ebdc1389f79c5932cd58d62 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xfc60f4d67ee85d99c7438783192a6afe . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x1d89d453f6d6c68499126f5b798f64b0 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xeab7e98a4954061f100c89f374ed62f8 . "./code.cg2js/cg-text-or-combo.fasl")
(#xe1d8846cbf313f9f9314eabfd11f6cde . "./code.cg2js/cg-text-widget.fasl")
(#xd7ac77823bee898424c6f300a07668e6 . "./code.cg2js/cg-timer.fasl")
(#x336efd1468a694887d7acbf9238092fe . "./code.cg2js/cg-toggling-widget.fasl")
(#x76b7ea5d357d6e7534f34da614239a91 . "./code.cg2js/cg-toolbar.fasl")
(#x1034ca92db30bf193c6c263f2f67a2a4 . "./code.cg2js/cg-tooltip.fasl")
(#x06113d18ec94912cbc63978a57c54816 . "./code.cg2js/cg-touch.fasl")
(#x16aacd1017ff2e84a16bc4dbd8f96e7d . "./code.cg2js/cg-trackbar.fasl")
(#xbc932107d4bed56f29851a2aeeea1f67 . "./code.cg2js/cg-tray.fasl")
(#xfc6886c04124e0a0eb688783fd3f9d53 . "./code.cg2js/cg-up-down-control.fasl")
(#x805a5cce055b48da19f7f6cfb97f73b8 . "./code.cg2js/cg-utility-dialog.fasl")
(#xb0079a39d04903bf989e115837b55a76 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x4be5e4261f65342d535ee2a3350f6d82 . "./code.cg2js/cg-web-browser.fasl")
(#x940d87c2138eb5ba5293e75f293b8fc8 . "./code.cg2js/cg-wrap-string.fasl")
(#x0567bfb411a7cd9b4550f9155330cc58 . "./code.cg2js/cg-yes-no-list.fasl")
(#x38f458e193f9fb72b0cdae4ad5f2eec7 . "./code.cg2js/cg-yes-no-string.fasl")
(#x6e523ef8a750c080dda7e4d3255eaa99 . "./code.cg2js/cg.fasl")
(#xc3b2dba3509e161a1980d4d357705f0d . "./code.idejs/ide-acache.fasl")
(#x4ccb244d8a80e8d3e5390590e9cff6ff . "./code.idejs/ide-apropos.fasl")
(#x33cbca879cdc122439e50bdc17e0d31e . "./code.idejs/ide-base.fasl")
(#x4c429c59a7c66c6fa655e850497983dd . "./code.idejs/ide-class-browser.fasl")
(#x7045bcf8d820b1c902fc5adaaba8ba86 . "./code.idejs/ide-class-grapher.fasl")
(#x5bacd51da18533e36f46bf16283aa6fa . "./code.idejs/ide-clipboard.fasl")
(#x26ca8f20fffaa748c3b9ca37f67793d7 . "./code.idejs/ide-dev.fasl")
(#x0b635e8c1e482bd9442b37de515e9c6e . "./code.idejs/ide-editor.fasl")
(#x8332d4b4d33917184fa6f8a5cff955ab . "./code.idejs/ide-find-definitions.fasl")
(#xaf93256661957f90383b401b17132c95 . "./code.idejs/ide-find-in-files.fasl")
(#xa6cd16210d81246e5e7c8aea2216114a . "./code.idejs/ide-form.fasl")
(#xeaf97670a29ab54bcb8643249575bf01 . "./code.idejs/ide-grapher.fasl")
(#xc2537174e92479bf7a98759703fb9255 . "./code.idejs/ide-inspector.fasl")
(#xe2ef45de1750df37d66c4e9eb8310994 . "./code.idejs/ide-intro.fasl")
(#xa40b560926f46162fb66078b841d3564 . "./code.idejs/ide-keyshort.fasl")
(#x32b4db88f020d809f2e240ae3f99042f . "./code.idejs/ide-navigator.fasl")
(#xacde481ec0abbdbcaf2f53de3fc260b5 . "./code.idejs/ide-package-list.fasl")
(#xac5baebee994d119ea335a86d5ce2d54 . "./code.idejs/ide-patch-dialog.fasl")
(#x22fb2095731a31cd6441577806c56dce . "./code.idejs/ide-process-browser.fasl")
(#x398c5cdcb0ac56a896096e96ed456c14 . "./code.idejs/ide-project.fasl")
(#x8be36a346f97accec79d600c448c1629 . "./code.idejs/ide-trace.fasl")
(#x3db28b5fedd92c377c2efb8443b75add . "./code.idejs/ide-window-list.fasl")
(#xfec607b7bb9f3e09f2447d2cc04583d3 . "./code.idejs/ide.fasl")
(#xc2912bce53894841a1e9ff8f0fd00d64 . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
